package betclic.test.configuration

import betclic.test.player.PlayerEntity
import dev.andrewohara.dynamokt.DataClassTableSchema
import io.ktor.server.application.*
import kotlinx.coroutines.future.await
import kotlinx.coroutines.runBlocking
import org.koin.ktor.ext.inject
import org.slf4j.LoggerFactory
import software.amazon.awssdk.enhanced.dynamodb.DynamoDbEnhancedAsyncClient
import software.amazon.awssdk.services.dynamodb.DynamoDbAsyncClient
import kotlin.reflect.KClass

fun Application.migrateTables() {
    val dynamoDbMigration: DynamoDbMigration by inject()
    val dynamoDbClient: DynamoDbAsyncClient by inject()
    val dynamoDbEnhancedClient: DynamoDbEnhancedAsyncClient by inject()

    runBlocking {
        dynamoDbMigration.createNecessaryTables(dynamoDbClient, dynamoDbEnhancedClient)
    }
}

class DynamoDbMigration {
    private val logger = LoggerFactory.getLogger(Application::class.java)

    suspend fun createNecessaryTables(
        dynamoDbClient: DynamoDbAsyncClient,
        dynamoDbEnhancedClient: DynamoDbEnhancedAsyncClient
    ) {
        val existingTables = dynamoDbClient.listTables().await().tableNames().toList()

        listOf(PlayerEntity::class).forEach {
            createTableIfNotExists(existingTables, it, dynamoDbEnhancedClient)
        }
    }


    private suspend fun <T : Any> createTableIfNotExists(
        existingTables: List<String>,
        item: KClass<T>,
        dynamoDbEnhancedClient: DynamoDbEnhancedAsyncClient
    ) {
        val tableSchema = DataClassTableSchema(item)
        if (existingTables.contains(item.simpleName)) {
            logger.info("Table '${item.simpleName}' already exists.")
        } else {
            dynamoDbEnhancedClient.table(item.simpleName, tableSchema).createTable().await()
            logger.info("Table '${item.simpleName}' created successfully.")
        }
    }
}

