package betclic.test.configuration

import betclic.test.player.PlayerRepository
import betclic.test.player.PlayerService
import betclic.test.player.PlayerServiceImpl
import io.ktor.server.application.*
import org.koin.dsl.module
import org.koin.ktor.plugin.Koin
import org.koin.logger.slf4jLogger

fun Application.configureKoin() {
    install(Koin) {
        slf4jLogger()
        modules(databaseModule, playerModule)
    }
}

val databaseModule = module {
    single { DynamoDbConfiguration().createDynamoDbClient() }
    single { DynamoDbConfiguration().createDataSource(get()) }
    single { DynamoDbMigration() }
}

val playerModule = module {
    single<PlayerRepository> { PlayerRepository(get()) }
    single<PlayerService> { PlayerServiceImpl(get()) }
}
