package betclic.test.player

import dev.andrewohara.dynamokt.DataClassTableSchema
import io.ktor.server.application.*
import kotlinx.coroutines.coroutineScope
import kotlinx.coroutines.future.await
import kotlinx.coroutines.reactive.asFlow
import org.slf4j.LoggerFactory
import software.amazon.awssdk.enhanced.dynamodb.DynamoDbEnhancedAsyncClient
import software.amazon.awssdk.enhanced.dynamodb.Key

class PlayerRepository(dynamoDbEnhancedClient: DynamoDbEnhancedAsyncClient) {
    private val tableName = PlayerEntity::class.simpleName
    private val tableSchema = DataClassTableSchema(PlayerEntity::class)
    private val table = dynamoDbEnhancedClient.table(tableName, tableSchema)
    private val logger = LoggerFactory.getLogger(Application::class.java)

    suspend fun createNewPlayer(player: Player) = coroutineScope {
        val createdPlayer = table.putItem(player.toPlayerEntity()).await()
        logger.info("Successfully created new player $createdPlayer")
    }

    suspend fun updatePlayer(player: Player): Player {
        val updatedPlayer = table.updateItem(player.toPlayerEntity()).await()
        return updatedPlayer.toPlayer()
    }

    suspend fun findPlayerByPseudo(pseudo: String): Player? {
        val foundPlayer = table.getItem(
            Key.builder().partitionValue(pseudo).build()
        ).await()
        return foundPlayer?.toPlayer()
    }

    suspend fun findAll(): List<Player> {
        return buildList {
            table.scan().asFlow().collect { it.items().stream().forEach { item -> add(item.toPlayer()) } }
        }
    }


    suspend fun deleteAllPlayers() {
        table.deleteTable()
        table.createTable().await()

    }
}

