package betclic.test.player

import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import org.koin.ktor.ext.inject

fun Routing.playerRoutes() {
    val playerService by inject<PlayerService>()
    // const
    route("/players") {
        post {
            // todo extract
            val request = call.receive<String>()
            // dto
            playerService.createNewPlayer(request)
            call.respond(HttpStatusCode.Created)
        }

        put {
            val request = call.receive<Player>()
            call.respond(playerService.updatePlayer(request))
        }

        get("/{pseudo}") {
            val pseudo = call.parameters["pseudo"] ?: return@get call.respond(HttpStatusCode.BadRequest)
            call.respond<Player>(playerService.findPlayerByPseudo(pseudo))
        }

        get("/{pseudo}/info") {
            val pseudo = call.parameters["pseudo"] ?: return@get call.respond(HttpStatusCode.BadRequest)
            call.respond<PlayerInfoDTO>(playerService.getPlayerInfoByPseudo(pseudo))
        }

        get("/ranking") {
            call.respond(playerService.getPlayersRanked())
        }

        delete {
            playerService.deleteAllPlayers()
            call.respond(HttpStatusCode.NoContent)
        }
    }

}