package betclic.test.player

import io.ktor.server.plugins.*

class PlayerServiceImpl(private val playerRepository: PlayerRepository) : PlayerService {

    override suspend fun createNewPlayer(pseudo: String) {
        playerRepository.createNewPlayer(Player(pseudo = pseudo))
    }

    override suspend fun updatePlayer(player: Player): Player {
        return playerRepository.updatePlayer(player)
    }

    override suspend fun findPlayerByPseudo(pseudo: String): Player {
        val player = playerRepository.findPlayerByPseudo(pseudo) ?: throw NotFoundException("Player $pseudo not found")
        return player
    }

    override suspend fun getPlayerInfoByPseudo(pseudo: String): PlayerInfoDTO {
        val allPlayersRanked = getPlayersRanked()
        val player = allPlayersRanked.find { it.pseudo == pseudo }
            ?: throw NotFoundException("Player $pseudo not found in ranking")
        return player
    }

    override suspend fun getPlayersRanked(): List<PlayerInfoDTO> {
        val allPlayers = playerRepository.findAll()
        return allPlayers.sortedByDescending { it.pointsNumber }.mapIndexed { index, player ->
            PlayerInfoDTO(
                pseudo = player.pseudo,
                pointsNumber = player.pointsNumber,
                ranking = index + 1
            )
        }

    }

    override suspend fun deleteAllPlayers() {
        playerRepository.deleteAllPlayers()
    }
}